@@ -11,7 +11,7 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized );
 
-our $VERSION = '1.833';
+our $VERSION = '1.838';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (Subroutines::ProhibitSubroutinePrototypes)
@@ -35,7 +35,7 @@ BEGIN {
    $_tid = $_has_threads ? threads->tid() : 0;
    $_oid = "$$.$_tid";
 
-   if ($] ge '5.008008' && !exists $INC{'PDL.pm'}) {
+   if ($] ge '5.008008' && !$INC{'PDL.pm'}) {
       eval '
          use Sereal::Encoder 3.015 qw( encode_sereal );
          use Sereal::Decoder 3.015 qw( decode_sereal );
@@ -59,6 +59,7 @@ BEGIN {
    return;
 }
 
+use IO::Handle ();
 use Scalar::Util qw( looks_like_number refaddr weaken );
 use Socket qw( SOL_SOCKET SO_RCVBUF );
 use Time::HiRes qw( sleep time );
@@ -196,9 +197,6 @@ sub import {
 
    return if $_imported++;
 
-   no strict 'refs'; no warnings 'redefine';
-   *{ 'MCE::_parse_max_workers' } = \&MCE::Util::_parse_max_workers;
-
    ## Instantiate a module-level instance.
    $MCE = MCE->new( _module_instance => 1, max_workers => 0 );
 
@@ -359,6 +357,18 @@ sub _restore_state {
 ##
 ###############################################################################
 
+sub _croak {
+   if (MCE->wid == 0 || ! $^S) {
+      $SIG{__DIE__}  = \&MCE::Signal::_die_handler;
+      $SIG{__WARN__} = \&MCE::Signal::_warn_handler;
+   }
+   $\ = undef; goto &Carp::croak;
+}
+
+use MCE::Core::Validation ();
+use MCE::Core::Manager ();
+use MCE::Core::Worker ();
+
 sub new {
    my ($class, %self) = @_;
    my $_pkg = exists $self{pkg} ? delete $self{pkg} : caller;
@@ -447,10 +457,6 @@ sub new {
       }
    }
 
-   require MCE::Core::Validation unless $INC{'MCE/Core/Validation.pm'};
-   require MCE::Core::Manager    unless $INC{'MCE/Core/Manager.pm'};
-   require MCE::Core::Worker     unless $INC{'MCE/Core/Worker.pm'};
-
    _validate_args(\%self);
 
    ## -------------------------------------------------------------------------
@@ -471,13 +477,13 @@ sub new {
    $self{_last_sref}  = (ref $self{input_data} eq 'SCALAR')
       ? refaddr($self{input_data}) : 0;
 
-   my $_data_channels = ($_oid eq "$$.$_tid") ? DATA_CHANNELS : 2;
+   my $_data_channels = ("$$.$_tid" eq $_oid) ? DATA_CHANNELS : 2;
    my $_total_workers = 0;
 
    if (defined $self{user_tasks}) {
       $_total_workers += $_->{max_workers} for (@{ $self{user_tasks} });
    } else {
-      $_total_workers  = $self{max_workers};
+      $_total_workers = $self{max_workers};
    }
 
    $self{_init_total_workers} = $_total_workers;
@@ -486,7 +492,7 @@ sub new {
       ? $_total_workers : $_data_channels;
 
    $self{_lock_chn} = ($_total_workers > $_data_channels) ? 1 : 0;
-   $self{_lock_chn} = 1 if ($INC{'MCE/Hobo.pm'});
+   $self{_lock_chn} = 1 if $INC{'MCE/Hobo.pm'};
 
    $MCE = \%self if ($MCE->{_wid} == 0);
 
@@ -513,12 +519,13 @@ sub spawn {
    lock $_MCE_LOCK if $_has_threads;  # Obtain locks
    lock $_WIN_LOCK if $_is_MSWin32;
 
-   sleep 0.015 if ($_tid && !$self->{use_threads});
-
    if ($INC{'PDL.pm'}) { local $@;
       eval 'use PDL::IO::Storable' unless $INC{'PDL/IO/Storable.pm'};
       eval 'PDL::no_clone_skip_warning()';
    }
+   if ( $INC{'LWP/UserAgent.pm'} && !$INC{'Net/HTTP.pm'} ) {
+      local $@; eval 'require Net::HTTP; require Net::HTTPS';
+   }
 
    ## Start the shared-manager process if present.
    MCE::Shared->start() if $INC{'MCE/Shared.pm'};
@@ -544,6 +551,10 @@ sub spawn {
       }
    }
 
+   if ("$$.$_tid" ne $_oid && (!$self->{use_threads} || $_is_MSWin32)) {
+      sleep 0.015;
+   }
+
    my $_die_handler  = $SIG{__DIE__};
    my $_warn_handler = $SIG{__WARN__};
 
@@ -568,7 +579,7 @@ sub spawn {
          my $_DAT_W_SOCK = $TOP_HDLR->{_dat_w_sock}->[0];
          print {$_DAT_W_SOCK} OUTPUT_S_IPC.$LF . '0'.$LF;
 
-         1 until sysread($_DAT_W_SOCK, my($_buf), 1) || ($! && !$!{'EINTR'});
+         MCE::Util::_sysread($_DAT_W_SOCK, my($_buf), 1);
       }
    }
 
@@ -601,7 +612,7 @@ sub spawn {
       : MCE::Util::_sock_pair($self, qw(_que_r_sock _que_w_sock));
 
    if (defined $self->{init_relay}) {                               # relay
-      unless (exists $INC{'MCE/Relay.pm'}) {
+      unless ($INC{'MCE/Relay.pm'}) {
          require MCE::Relay; MCE::Relay->import();
       }
       MCE::Util::_sock_pair($self, qw(_rla_r_sock _rla_w_sock), $_)
@@ -616,6 +627,10 @@ sub spawn {
    $self->{_pids}   = [], $self->{_thrs}  = [], $self->{_tids} = [];
    $self->{_status} = [], $self->{_state} = [], $self->{_task} = [];
 
+   local $SIG{TTIN}  unless $_is_MSWin32;
+   local $SIG{TTOU}  unless $_is_MSWin32;
+   local $SIG{WINCH} unless $_is_MSWin32;
+
    if (!defined $self->{user_tasks}) {
       $self->{_total_workers} = $_max_workers;
 
@@ -747,7 +762,10 @@ sub relay (;&) {
       unless (defined $MCE->{init_relay});
 }
 
-*relay_unlock = \&relay;
+{
+   no warnings 'once';
+   *relay_unlock = \&relay;
+}
 
 sub AUTOLOAD {
    # $AUTOLOAD = MCE::<method_name>
@@ -1105,9 +1123,9 @@ sub run {
 
       ## Insert the first message into the queue if defined.
       if (defined $_first_msg) {
-         1 until syswrite (
+         MCE::Util::_syswrite(
             $self->{_que_w_sock}, pack($_que_template, 0, $_first_msg)
-         ) || ($! && !$!{'EINTR'});
+         );
       }
 
       ## Submit params data to workers.
@@ -1144,11 +1162,11 @@ sub run {
          ## Notify workers to commence processing.
          if ($_is_MSWin32) {
             my $_buf = _sprintf("%${_total_workers}s", "");
-            syswrite $self->{_bse_w_sock}, $_buf;
+            MCE::Util::_syswrite($self->{_bse_w_sock}, $_buf);
          } else {
             my $_BSE_W_SOCK = $self->{_bse_w_sock};
             for my $_i (1 .. $_total_workers) {
-               1 until syswrite($_BSE_W_SOCK, $LF) || ($! && !$!{'EINTR'});
+               MCE::Util::_syswrite($_BSE_W_SOCK, $LF);
             }
          }
       }
@@ -1162,11 +1180,8 @@ sub run {
 
    ## Remove the last message from the queue.
    if (!$_send_cnt && $_run_mode ne 'nodata') {
-      if (defined $self->{_que_r_sock}) {
-         1 until sysread (
-            $self->{_que_r_sock}, my($_buf), $_que_read_size
-         ) || ($! && !$!{'EINTR'});
-      }
+      MCE::Util::_sysread($self->{_que_r_sock}, my($_buf), $_que_read_size)
+         if ( defined $self->{_que_r_sock} );
    }
 
    $self->{_send_cnt} = 0;
@@ -1387,14 +1402,14 @@ sub sync {
    print {$_DAT_W_SOCK} OUTPUT_B_SYN.$LF . $_chn.$LF;
 
    ## Wait until all workers from (task_id 0) have synced.
-   MCE::Util::_sock_ready($_BSB_R_SOCK) if $_is_MSWin32;
-   1 until sysread($_BSB_R_SOCK, $_buf, 1) || ($! && !$!{'EINTR'});
+   MCE::Util::_sock_ready($_BSB_R_SOCK, -1) if $_is_MSWin32;
+   MCE::Util::_sysread($_BSB_R_SOCK, $_buf, 1);
 
    ## Notify the manager process (barrier end).
    print {$_DAT_W_SOCK} OUTPUT_E_SYN.$LF . $_chn.$LF;
 
    ## Wait until all workers from (task_id 0) have un-synced.
-   1 until sysread($_BSE_R_SOCK, $_buf, 1) || ($! && !$!{'EINTR'});
+   MCE::Util::_sysread($_BSE_R_SOCK, $_buf, 1);
 
    return;
 }
@@ -1444,12 +1459,8 @@ sub abort {
       local $\ = undef;
 
       if ($_abort_msg > 0) {
-         1 until sysread (
-            $_QUE_R_SOCK, my($_next), $_que_read_size
-         ) || ($! && !$!{'EINTR'});
-         1 until syswrite (
-            $_QUE_W_SOCK, pack($_que_template, 0, $_abort_msg)
-         ) || ($! && !$!{'EINTR'});
+         MCE::Util::_sysread($_QUE_R_SOCK, my($_next), $_que_read_size);
+         MCE::Util::_syswrite($_QUE_W_SOCK, pack($_que_template, 0, $_abort_msg));
       }
 
       if ($self->{_wid} > 0) {
@@ -1771,15 +1782,6 @@ sub say {
 ##
 ###############################################################################
 
-sub _croak {
-   if (MCE->wid == 0 || ! $^S) {
-      $SIG{__DIE__}  = \&MCE::Signal::_die_handler;
-      $SIG{__WARN__} = \&MCE::Signal::_warn_handler;
-   }
-
-   $\ = undef; goto &Carp::croak;
-}
-
 sub _exit {
    my $self = shift;
 
@@ -1791,11 +1793,13 @@ sub _exit {
 
    threads->exit(0) if $self->{use_threads};
 
-   $SIG{HUP} = $SIG{INT} = $SIG{QUIT} = $SIG{TERM} = sub {
-      $SIG{$_[0]} = $SIG{INT} = sub { };
-      CORE::kill($_[0], getppid()) if ($_[0] eq 'INT' && !$_is_MSWin32);
-      CORE::kill('KILL', $$);
-   };
+   if (! $_tid) {
+      $SIG{HUP} = $SIG{INT} = $SIG{QUIT} = $SIG{TERM} = sub {
+         $SIG{$_[0]} = $SIG{INT} = sub { };
+         CORE::kill($_[0], getppid()) if ($_[0] eq 'INT' && !$_is_MSWin32);
+         CORE::kill('KILL', $$);
+      };
+   }
 
    if ($self->{posix_exit} && !$_is_MSWin32) {
       eval { MCE::Mutex::Channel::_destroy() };
@@ -1928,8 +1932,8 @@ sub _dispatch {
    ## Sets the seed of the base generator uniquely between workers.
    ## The new seed is computed using the current seed and $_wid value.
    ## One may set the seed at the application level for predictable
-   ## results (non-thread workers only). Ditto for Math::Prime::Util
-   ## and Math::Random.
+   ## results (non-thread workers only). Ditto for Math::Prime::Util,
+   ## Math::Random, and Math::Random::MT::Auto.
 
    if (!$self->{use_threads}) {
       my ($_wid, $_seed) = ($_args[1], $self->{_seed});
@@ -1942,16 +1946,27 @@ sub _dispatch {
          if ( $INC{'MCE/Hobo.pm'} && MCE::Hobo->can('_clear') );
    }
 
-   if ($INC{'Math/Random.pm'} && !$self->{use_threads}) {
+   if (!$self->{use_threads} && $INC{'Math/Random.pm'}) {
       my ($_wid, $_cur_seed) = ($_args[1], Math::Random::random_get_seed());
 
       my $_new_seed = ($_cur_seed < 1073741781)
-         ? $_cur_seed + ($_wid * 100000)
-         : $_cur_seed - ($_wid * 100000);
+         ? $_cur_seed + (($_wid * 100000) % 1073741780)
+         : $_cur_seed - (($_wid * 100000) % 1073741780);
 
       Math::Random::random_set_seed($_new_seed, $_new_seed);
    }
 
+   if (!$self->{use_threads} && $INC{'Math/Random/MT/Auto.pm'}) {
+      my ($_wid, $_cur_seed) = (
+         $_args[1], Math::Random::MT::Auto::get_seed()->[0]
+      );
+      my $_new_seed = ($_cur_seed < 1073741781)
+         ? $_cur_seed + (($_wid * 100000) % 1073741780)
+         : $_cur_seed - (($_wid * 100000) % 1073741780);
+
+      Math::Random::MT::Auto::set_seed($_new_seed);
+   }
+
    ## Run.
 
    $self->{_pid} = ($_is_thread) ? $$ .'.'. threads->tid() : $$;
